<?php
/*
Plugin Name: Patreon User Role Changer
Description: Changes the role of newly created users with a username starting with "patreon_" to a configurable role.
Version: 1.0.4.1
Author: Tyson Brooks
Author URI: https://frostlineworks.com
*/



if (!defined('ABSPATH')) {
    exit;
}
// Check if the FLW Plugin Library is active
if (class_exists('FLW_Plugin_Update_Checker')) {
    $pluginSlug = basename(dirname(__FILE__)); // Dynamically get the plugin slug

    // Initialize the update checker
    FLW_Plugin_Update_Checker::initialize(__FILE__, $pluginSlug);

    // Replace the update icon
    add_filter('site_transient_update_plugins', function ($transient) {
        if (isset($transient->response)) {
            foreach ($transient->response as $plugin_slug => $plugin_data) {
                // Use the plugin's main file path to determine its asset URL
                if ($plugin_slug === plugin_basename(__FILE__)) {
                    $icon_url = plugins_url('assets/logo-128x128.png', __FILE__);
                    $transient->response[$plugin_slug]->icons = [
                        'default' => $icon_url,
                        '1x' => $icon_url,
                        '2x' => plugins_url('assets/logo-256x256.png', __FILE__),
                    ];
                }
            }
        }
        return $transient;
    });
} else {
    // Admin notice for missing library
    add_action('admin_notices', function () {
        $pluginSlug = 'flwpluginlibrary/flwpluginlibrary.php';
        $plugins = get_plugins();

        if (!isset($plugins[$pluginSlug])) {
            echo '<div class="notice notice-error"><p>The FLW Plugin Library is not installed. Please install and activate it to enable update functionality.</p></div>';
        } elseif (!is_plugin_active($pluginSlug)) {
            $activateUrl = wp_nonce_url(
                admin_url('plugins.php?action=activate&plugin=' . $pluginSlug),
                'activate-plugin_' . $pluginSlug
            );
            echo '<div class="notice notice-error"><p>The FLW Plugin Library is installed but not active. Please <a href="' . esc_url($activateUrl) . '">activate</a> it to enable update functionality.</p></div>';
        }
    });
}




// Add a "Settings" link to the plugin actions in the plugins list
add_filter('plugin_action_links_' . plugin_basename(__FILE__), function ($links) {
    $settings_link = '<a href="admin.php?page=patreon-user-role-settings">Settings</a>';
    array_unshift($links, $settings_link); // Add the settings link at the beginning
    return $links;
});

// Function to clone the Subscriber role and create the Patreon Member role.
function clone_subscriber_role() {
    // Get the Subscriber role.
    $subscriber_role = get_role('subscriber');

    // Check if the Subscriber role exists and the Patreon Member role does not already exist.
    if ($subscriber_role && !get_role('patreon_member')) {
        // Add the Patreon Member role with the same capabilities as Subscriber.
        add_role(
            'patreon_member', // Role slug.
            'Patreon Member', // Display name.
            $subscriber_role->capabilities // Capabilities.
        );
    }
}

// Hook into plugin activation to create the new role.
register_activation_hook(__FILE__, 'clone_subscriber_role');

class Patreon_User_Role_Changer {

    private $option_name = 'patreon_user_role';

    public function __construct() {
        add_action('user_register', [$this, 'change_user_role'], 10, 1);
        add_action('admin_menu', [$this, 'add_settings_page']);
        add_action('admin_init', [$this, 'register_settings']);
    }

    /**
     * Changes the role of users with username starting with 'patreon_'.
     *
     * @param int $user_id The ID of the newly registered user.
     */
    public function change_user_role($user_id) {
        $user = get_userdata($user_id);
        if (!$user) {
            return;
        }

        // Check if the username starts with 'patreon_'
        if (strpos($user->user_login, 'patreon_') === 0) {
            // Get the configured role
            $role = get_option($this->option_name, 'subscriber');

            // Ensure the role exists
            if (get_role($role)) {
                $user->set_role($role);
            }
        }
    }

    /**
     * Adds a settings page under the "FLW Plugins" menu.
     */
    public function add_settings_page() {
        add_submenu_page(
            'flw-plugins',             // Parent slug (ties to the FLW Plugins menu)
            'Patreon User Role Settings', // Page title
            'Patreon Role',            // Submenu title
            'manage_options',          // Capability required
            'patreon-user-role-settings', // Menu slug
            [$this, 'render_settings_page'] // Callback function
        );
    }


    /**
     * Registers the plugin settings.
     */
    public function register_settings() {
        register_setting('patreon_user_role_settings', $this->option_name, [
            'type' => 'string',
            'sanitize_callback' => [$this, 'sanitize_role'],
            'default' => 'subscriber',
        ]);
    }

    /**
     * Sanitizes the role input.
     *
     * @param string $role The role selected by the admin.
     * @return string Sanitized role.
     */
    public function sanitize_role($role) {
        return get_role($role) ? $role : 'subscriber';
    }

    /**
     * Renders the settings page.
     */
    public function render_settings_page() {
        global $wp_roles;
        $roles = $wp_roles->roles;
        $selected_role = get_option($this->option_name, 'subscriber');
        ?>
        <div class="wrap">
            <h1>Patreon User Role Settings</h1>
            <form method="post" action="options.php">
                <?php
                settings_fields('patreon_user_role_settings');
                do_settings_sections('patreon_user_role_settings');
                ?>
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row">Role for Patreon Members</th>
                        <td>
                            <select name="<?php echo esc_attr($this->option_name); ?>">
                                <?php foreach ($roles as $role_key => $role_data) : ?>
                                    <option value="<?php echo esc_attr($role_key); ?>" <?php selected($selected_role, $role_key); ?>>
                                        <?php echo esc_html($role_data['name']); ?>
                                    </option>
                                <?php endforeach; ?>
                            </select>
                            <p class="description">Select the role to assign to new Patreon users.</p>
                        </td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div>
        <?php
    }
}

// Initialize the plugin
new Patreon_User_Role_Changer();